import JWT from "../../utils/JWT.ts";
import HttpError from "../HttpError.ts";
import Router, { makeParameterModifier } from "../Router.ts";

export default function jwt<T extends Router>(salt: ((self: T) => string) | keyof T, required = false) {
    return makeParameterModifier<T>(function (_req, val?: string) {
        if (val === undefined) return undefined;
        const s = typeof salt === 'function' ? salt(this) : this[salt] as string;
        try {
            const res = JWT.decode(val, s);
            if (required && res === undefined) throw new HttpError('You are not logged in.');
            return res;
        }
        catch (e) {
            if (e instanceof Error && !(e instanceof HttpError)) {
                throw new HttpError(e.message, 400);
            }
            else throw e;
        }
    });
}